@@ -25,9 +25,6 @@
 
 #define NONAMELESSUNION
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -42,7 +39,6 @@
 #include <commdlg.h>
 #include <cpl.h>
 
-#include "wine/unicode.h"
 #include "wine/list.h"
 #include "wine/debug.h"
 #include "appwiz.h"
@@ -67,6 +63,12 @@ typedef struct APPINFO
 
     LPWSTR publisher;
     LPWSTR version;
+    LPWSTR contact;
+    LPWSTR helplink;
+    LPWSTR helptelephone;
+    LPWSTR readme;
+    LPWSTR urlupdateinfo;
+    LPWSTR comments;
 
     HKEY regroot;
     WCHAR regkey[MAX_STRING_LEN];
@@ -84,21 +86,17 @@ static const WCHAR openW[] = {'o','p','e','n',0};
 static const WCHAR BackSlashW[] = { '\\', 0 };
 static const WCHAR DisplayNameW[] = {'D','i','s','p','l','a','y','N','a','m','e',0};
 static const WCHAR DisplayIconW[] = {'D','i','s','p','l','a','y','I','c','o','n',0};
-static const WCHAR DisplayVersionW[] = {'D','i','s','p','l','a','y','V','e','r',
-    's','i','o','n',0};
+static const WCHAR DisplayVersionW[] = {'D','i','s','p','l','a','y','V','e','r','s','i','o','n',0};
 static const WCHAR PublisherW[] = {'P','u','b','l','i','s','h','e','r',0};
 static const WCHAR ContactW[] = {'C','o','n','t','a','c','t',0};
 static const WCHAR HelpLinkW[] = {'H','e','l','p','L','i','n','k',0};
-static const WCHAR HelpTelephoneW[] = {'H','e','l','p','T','e','l','e','p','h',
-    'o','n','e',0};
+static const WCHAR HelpTelephoneW[] = {'H','e','l','p','T','e','l','e','p','h','o','n','e',0};
 static const WCHAR ModifyPathW[] = {'M','o','d','i','f','y','P','a','t','h',0};
 static const WCHAR NoModifyW[] = {'N','o','M','o','d','i','f','y',0};
 static const WCHAR ReadmeW[] = {'R','e','a','d','m','e',0};
-static const WCHAR URLUpdateInfoW[] = {'U','R','L','U','p','d','a','t','e','I',
-    'n','f','o',0};
+static const WCHAR URLUpdateInfoW[] = {'U','R','L','U','p','d','a','t','e','I','n','f','o',0};
 static const WCHAR CommentsW[] = {'C','o','m','m','e','n','t','s',0};
-static const WCHAR UninstallCommandlineW[] = {'U','n','i','n','s','t','a','l','l',
-    'S','t','r','i','n','g',0};
+static const WCHAR UninstallCommandlineW[] = {'U','n','i','n','s','t','a','l','l','S','t','r','i','n','g',0};
 static const WCHAR WindowsInstallerW[] = {'W','i','n','d','o','w','s','I','n','s','t','a','l','l','e','r',0};
 static const WCHAR SystemComponentW[] = {'S','y','s','t','e','m','C','o','m','p','o','n','e','n','t',0};
 
@@ -139,9 +137,27 @@ static void FreeAppInfo(APPINFO *info)
     HeapFree(GetProcessHeap(), 0, info->icon);
     HeapFree(GetProcessHeap(), 0, info->publisher);
     HeapFree(GetProcessHeap(), 0, info->version);
+    HeapFree(GetProcessHeap(), 0, info->contact);
+    HeapFree(GetProcessHeap(), 0, info->helplink);
+    HeapFree(GetProcessHeap(), 0, info->helptelephone);
+    HeapFree(GetProcessHeap(), 0, info->readme);
+    HeapFree(GetProcessHeap(), 0, info->urlupdateinfo);
+    HeapFree(GetProcessHeap(), 0, info->comments);
     HeapFree(GetProcessHeap(), 0, info);
 }
 
+static WCHAR *get_reg_str(HKEY hkey, const WCHAR *value)
+{
+    DWORD len, type;
+    WCHAR *ret = NULL;
+    if (!RegQueryValueExW(hkey, value, NULL, &type, NULL, &len) && type == REG_SZ)
+    {
+        if (!(ret = HeapAlloc(GetProcessHeap(), 0, len))) return NULL;
+        RegQueryValueExW(hkey, value, 0, 0, (BYTE *)ret, &len);
+    }
+    return ret;
+}
+
 /******************************************************************************
  * Name       : ReadApplicationsFromRegistry
  * Description: Creates a linked list of uninstallable applications from the
@@ -229,39 +245,23 @@ static BOOL ReadApplicationsFromRegistry(HKEY root)
                     &displen);
 
                 /* separate the index from the icon name, if supplied */
-                iconPtr = strchrW(info->icon, ',');
+                iconPtr = wcschr(info->icon, ',');
 
                 if (iconPtr)
                 {
                     *iconPtr++ = 0;
-                    info->iconIdx = atoiW(iconPtr);
+                    info->iconIdx = wcstol(iconPtr, NULL, 10);
                 }
             }
 
-            /* publisher, version */
-            if (RegQueryValueExW(hkeyApp, PublisherW, 0, 0, NULL, &displen) ==
-                ERROR_SUCCESS)
-            {
-                info->publisher = HeapAlloc(GetProcessHeap(), 0, displen);
-
-                if (!info->publisher)
-                    goto err;
-
-                RegQueryValueExW(hkeyApp, PublisherW, 0, 0, (LPBYTE)info->publisher,
-                    &displen);
-            }
-
-            if (RegQueryValueExW(hkeyApp, DisplayVersionW, 0, 0, NULL, &displen) ==
-                ERROR_SUCCESS)
-            {
-                info->version = HeapAlloc(GetProcessHeap(), 0, displen);
-
-                if (!info->version)
-                    goto err;
-
-                RegQueryValueExW(hkeyApp, DisplayVersionW, 0, 0, (LPBYTE)info->version,
-                    &displen);
-            }
+            info->publisher = get_reg_str(hkeyApp, PublisherW);
+            info->version = get_reg_str(hkeyApp, DisplayVersionW);
+            info->contact = get_reg_str(hkeyApp, ContactW);
+            info->helplink = get_reg_str(hkeyApp, HelpLinkW);
+            info->helptelephone = get_reg_str(hkeyApp, HelpTelephoneW);
+            info->readme = get_reg_str(hkeyApp, ReadmeW);
+            info->urlupdateinfo = get_reg_str(hkeyApp, URLUpdateInfoW);
+            info->comments = get_reg_str(hkeyApp, CommentsW);
 
             /* Check if NoModify is set */
             dwType = REG_DWORD;
@@ -455,7 +455,7 @@ static void InstallProgram(HWND hWnd)
     LoadStringW(hInst, IDS_FILTER_PROGRAMS, filter_programs, ARRAY_SIZE(filter_programs));
     LoadStringW(hInst, IDS_FILTER_ALL, filter_all, ARRAY_SIZE(filter_all));
 
-    snprintfW( FilterBufferW, MAX_PATH, filters, filter_installs, 0, 0,
+    swprintf( FilterBufferW, MAX_PATH, filters, filter_installs, 0, 0,
                filter_programs, 0, 0, filter_all, 0, 0 );
     memset(&ofn, 0, sizeof(OPENFILENAMEW));
     ofn.lStructSize = sizeof(OPENFILENAMEW);
@@ -616,28 +616,16 @@ static INT_PTR CALLBACK SupportInfoDlgProc(HWND hWnd, UINT msg, WPARAM wParam, L
                     RegOpenKeyExW(iter->regroot, key, 0, KEY_READ, &hkey);
 
                     /* Load our "not specified" string */
-                    LoadStringW(hInst, IDS_NOT_SPECIFIED, notfound,
-                        ARRAY_SIZE(notfound));
+                    LoadStringW(hInst, IDS_NOT_SPECIFIED, notfound, ARRAY_SIZE(notfound));
 
-                    /* Update the data for items already read into the structure */
-                    SetInfoDialogText(NULL, iter->publisher, notfound, hWnd,
-                        IDC_INFO_PUBLISHER);
-                    SetInfoDialogText(NULL, iter->version, notfound, hWnd,
-                        IDC_INFO_VERSION);
-
-                    /* And now update the data for those items in the registry */
-                    SetInfoDialogText(hkey, ContactW, notfound, hWnd,
-                        IDC_INFO_CONTACT);
-                    SetInfoDialogText(hkey, HelpLinkW, notfound, hWnd,
-                        IDC_INFO_SUPPORT);
-                    SetInfoDialogText(hkey, HelpTelephoneW, notfound, hWnd,
-                        IDC_INFO_PHONE);
-                    SetInfoDialogText(hkey, ReadmeW, notfound, hWnd,
-                        IDC_INFO_README);
-                    SetInfoDialogText(hkey, URLUpdateInfoW, notfound, hWnd,
-                        IDC_INFO_UPDATES);
-                    SetInfoDialogText(hkey, CommentsW, notfound, hWnd,
-                        IDC_INFO_COMMENTS);
+                    SetInfoDialogText(NULL, iter->publisher, notfound, hWnd, IDC_INFO_PUBLISHER);
+                    SetInfoDialogText(NULL, iter->version, notfound, hWnd, IDC_INFO_VERSION);
+                    SetInfoDialogText(hkey, iter->contact, notfound, hWnd, IDC_INFO_CONTACT);
+                    SetInfoDialogText(hkey, iter->helplink, notfound, hWnd, IDC_INFO_SUPPORT);
+                    SetInfoDialogText(hkey, iter->helptelephone, notfound, hWnd, IDC_INFO_PHONE);
+                    SetInfoDialogText(hkey, iter->readme, notfound, hWnd, IDC_INFO_README);
+                    SetInfoDialogText(hkey, iter->urlupdateinfo, notfound, hWnd, IDC_INFO_UPDATES);
+                    SetInfoDialogText(hkey, iter->comments, notfound, hWnd, IDC_INFO_COMMENTS);
 
                     /* Update the main label with the app name */
                     if (GetWindowTextW(GetDlgItem(hWnd, IDC_INFO_LABEL), oldtitle,
@@ -977,12 +965,12 @@ static LONG start_params(const WCHAR *params)
     if(!params)
         return FALSE;
 
-    if(!strcmpW(params, install_geckoW)) {
+    if(!wcscmp(params, install_geckoW)) {
         install_addon(ADDON_GECKO);
         return TRUE;
     }
 
-    if(!strcmpW(params, install_monoW)) {
+    if(!wcscmp(params, install_monoW)) {
         install_addon(ADDON_MONO);
         return TRUE;
     }
