@@ -101,14 +101,13 @@ static const char hex[16] = "0123456789ABCDEF";
  * Copy the content of an entry into a buffer, removing quotes, and possibly
  * translating environment variables.
  */
-static void PROFILE_CopyEntry( LPWSTR buffer, LPCWSTR value, int len,
-                               BOOL strip_quote )
+static void PROFILE_CopyEntry( LPWSTR buffer, LPCWSTR value, int len )
 {
     WCHAR quote = '\0';
 
     if(!buffer) return;
 
-    if (strip_quote && ((*value == '\'') || (*value == '\"')))
+    if (*value == '\'' || *value == '\"')
     {
         if (value[1] && (value[strlenW(value)-1] == *value)) quote = *value++;
     }
@@ -489,29 +488,6 @@ static PROFILESECTION *PROFILE_Load(HANDLE hFile, ENCODING * pEncoding)
 }
 
 
-/***********************************************************************
- *           PROFILE_DeleteSection
- *
- * Delete a section from a profile tree.
- */
-static BOOL PROFILE_DeleteSection( PROFILESECTION **section, LPCWSTR name )
-{
-    while (*section)
-    {
-        if (!strcmpiW( (*section)->name, name ))
-        {
-            PROFILESECTION *to_del = *section;
-            *section = to_del->next;
-            to_del->next = NULL;
-            PROFILE_Free( to_del );
-            return TRUE;
-        }
-        section = &(*section)->next;
-    }
-    return FALSE;
-}
-
-
 /***********************************************************************
  *           PROFILE_DeleteKey
  *
@@ -857,16 +833,26 @@ static BOOL PROFILE_Open( LPCWSTR filename, BOOL write_access )
  * Returns all keys of a section.
  * If return_values is TRUE, also include the corresponding values.
  */
-static INT PROFILE_GetSection( PROFILESECTION *section, LPCWSTR section_name,
+static INT PROFILE_GetSection( const WCHAR *filename, LPCWSTR section_name,
 			       LPWSTR buffer, UINT len, BOOL return_values )
 {
+    PROFILESECTION *section;
     PROFILEKEY *key;
 
     if(!buffer) return 0;
 
     TRACE("%s,%p,%u\n", debugstr_w(section_name), buffer, len);
 
-    while (section)
+    EnterCriticalSection( &PROFILE_CritSect );
+
+    if (!PROFILE_Open( filename, FALSE ))
+    {
+        LeaveCriticalSection( &PROFILE_CritSect );
+        buffer[0] = 0;
+        return 0;
+    }
+
+    for (section = CurProfile->section; section; section = section->next)
     {
         if (!strcmpiW( section->name, section_name ))
         {
@@ -877,19 +863,22 @@ static INT PROFILE_GetSection( PROFILESECTION *section, LPCWSTR section_name,
                 if (!*key->name && !key->value) continue;  /* Skip empty lines */
                 if (IS_ENTRY_COMMENT(key->name)) continue;  /* Skip comments */
                 if (!return_values && !key->value) continue;  /* Skip lines w.o. '=' */
-                PROFILE_CopyEntry( buffer, key->name, len - 1, 0 );
+                lstrcpynW( buffer, key->name, len - 1 );
                 len -= strlenW(buffer) + 1;
                 buffer += strlenW(buffer) + 1;
 		if (len < 2)
 		    break;
 		if (return_values && key->value) {
 			buffer[-1] = '=';
-			PROFILE_CopyEntry ( buffer, key->value, len - 1, 0 );
+                    lstrcpynW( buffer, key->value, len - 1 );
 			len -= strlenW(buffer) + 1;
 			buffer += strlenW(buffer) + 1;
                 }
             }
             *buffer = '\0';
+
+            LeaveCriticalSection( &PROFILE_CritSect );
+
             if (len <= 1)
                 /*If either lpszSection or lpszKey is NULL and the supplied
                   destination buffer is too small to hold all the strings,
@@ -902,12 +891,45 @@ static INT PROFILE_GetSection( PROFILESECTION *section, LPCWSTR section_name,
             }
             return oldlen - len;
         }
-        section = section->next;
     }
     buffer[0] = buffer[1] = '\0';
+
+    LeaveCriticalSection( &PROFILE_CritSect );
+
     return 0;
 }
 
+static BOOL PROFILE_DeleteSection( const WCHAR *filename, const WCHAR *name )
+{
+    PROFILESECTION **section;
+
+    EnterCriticalSection( &PROFILE_CritSect );
+
+    if (!PROFILE_Open( filename, TRUE ))
+    {
+        LeaveCriticalSection( &PROFILE_CritSect );
+        return FALSE;
+    }
+
+    for (section = &CurProfile->section; *section; section = &(*section)->next)
+    {
+        if (!strcmpiW( (*section)->name, name ))
+        {
+            PROFILESECTION *to_del = *section;
+            *section = to_del->next;
+            to_del->next = NULL;
+            PROFILE_Free( to_del );
+            CurProfile->changed = TRUE;
+            PROFILE_FlushFile();
+            break;
+        }
+    }
+
+    LeaveCriticalSection( &PROFILE_CritSect );
+    return TRUE;
+}
+
+
 /* See GetPrivateProfileSectionNamesA for documentation */
 static INT PROFILE_GetSectionNames( LPWSTR buffer, UINT len )
 {
@@ -949,64 +971,6 @@ static INT PROFILE_GetSectionNames( LPWSTR buffer, UINT len )
     return buf-buffer;
 }
 
-
-/***********************************************************************
- *           PROFILE_GetString
- *
- * Get a profile string.
- *
- * Tests with GetPrivateProfileString16, W95a,
- * with filled buffer ("****...") and section "set1" and key_name "1" valid:
- * section	key_name	def_val		res	buffer
- * "set1"	"1"		"x"		43	[data]
- * "set1"	"1   "		"x"		43	[data]		(!)
- * "set1"	"  1  "'	"x"		43	[data]		(!)
- * "set1"	""		"x"		1	"x"
- * "set1"	""		"x   "		1	"x"		(!)
- * "set1"	""		"  x   "	3	"  x"		(!)
- * "set1"	NULL		"x"		6	"1\02\03\0\0"
- * "set1"	""		"x"		1	"x"
- * NULL		"1"		"x"		0	""		(!)
- * ""		"1"		"x"		1	"x"
- * NULL		NULL		""		0	""
- *
- *
- */
-static INT PROFILE_GetString( LPCWSTR section, LPCWSTR key_name,
-                              LPCWSTR def_val, LPWSTR buffer, UINT len )
-{
-    PROFILEKEY *key = NULL;
-    static const WCHAR empty_strW[] = { 0 };
-
-    if(!buffer || !len) return 0;
-
-    if (!def_val) def_val = empty_strW;
-    if (key_name)
-    {
-        key = PROFILE_Find( &CurProfile->section, section, key_name, FALSE, FALSE);
-        PROFILE_CopyEntry( buffer, (key && key->value) ? key->value : def_val,
-                           len, TRUE );
-        TRACE("(%s,%s,%s): returning %s\n",
-              debugstr_w(section), debugstr_w(key_name),
-              debugstr_w(def_val), debugstr_w(buffer) );
-        return strlenW( buffer );
-    }
-    /* no "else" here ! */
-    if (section)
-    {
-        INT ret = PROFILE_GetSection(CurProfile->section, section, buffer, len, FALSE);
-        if (!buffer[0]) /* no luck -> def_val */
-        {
-            PROFILE_CopyEntry(buffer, def_val, len, TRUE);
-            ret = strlenW(buffer);
-        }
-        return ret;
-    }
-    buffer[0] = '\0';
-    return 0;
-}
-
-
 /***********************************************************************
  *           PROFILE_SetString
  *
@@ -1015,15 +979,7 @@ static INT PROFILE_GetString( LPCWSTR section, LPCWSTR key_name,
 static BOOL PROFILE_SetString( LPCWSTR section_name, LPCWSTR key_name,
                                LPCWSTR value, BOOL create_always )
 {
-    if (!key_name)  /* Delete a whole section */
-    {
-        TRACE("(%s)\n", debugstr_w(section_name));
-        CurProfile->changed |= PROFILE_DeleteSection( &CurProfile->section,
-                                                      section_name );
-        return TRUE;         /* Even if PROFILE_DeleteSection() has failed,
-                                this is not an error on application's level.*/
-    }
-    else if (!value)  /* Delete a key */
+    if (!value)  /* Delete a key */
     {
         TRACE("(%s,%s)\n", debugstr_w(section_name), debugstr_w(key_name) );
         CurProfile->changed |= PROFILE_DeleteKey( &CurProfile->section,
@@ -1087,45 +1043,57 @@ INT WINAPI GetPrivateProfileStringW( LPCWSTR section, LPCWSTR entry,
 				     LPCWSTR def_val, LPWSTR buffer,
 				     UINT len, LPCWSTR filename )
 {
+    static const WCHAR emptyW[] = {0};
     int		ret;
     LPWSTR	defval_tmp = NULL;
+    const WCHAR *p;
 
     TRACE("%s,%s,%s,%p,%u,%s\n", debugstr_w(section), debugstr_w(entry),
           debugstr_w(def_val), buffer, len, debugstr_w(filename));
 
-    /* strip any trailing ' ' of def_val. */
-    if (def_val)
+    if (!buffer || !len) return 0;
+    if (!def_val) def_val = emptyW;
+    if (!section) return GetPrivateProfileSectionNamesW( buffer, len, filename );
+    if (!entry)
     {
-        LPCWSTR p = def_val + strlenW(def_val) - 1;
-
-        while (p > def_val && *p == ' ')
-            p--;
-
-        if (p >= def_val)
+        ret = PROFILE_GetSection( filename, section, buffer, len, FALSE );
+        if (!buffer[0])
         {
-            int vlen = (int)(p - def_val) + 1;
-
-            defval_tmp = HeapAlloc(GetProcessHeap(), 0, (vlen + 1) * sizeof(WCHAR));
-            memcpy(defval_tmp, def_val, vlen * sizeof(WCHAR));
-            defval_tmp[vlen] = '\0';
-            def_val = defval_tmp;
+            PROFILE_CopyEntry( buffer, def_val, len );
+            ret = strlenW( buffer );
         }
+        return ret;
+    }
+
+    /* strip any trailing ' ' of def_val. */
+    p = def_val + strlenW(def_val) - 1;
+
+    while (p > def_val && *p == ' ') p--;
+
+    if (p >= def_val)
+    {
+        int vlen = (int)(p - def_val) + 1;
+
+        defval_tmp = HeapAlloc(GetProcessHeap(), 0, (vlen + 1) * sizeof(WCHAR));
+        memcpy(defval_tmp, def_val, vlen * sizeof(WCHAR));
+        defval_tmp[vlen] = '\0';
+        def_val = defval_tmp;
     }
 
     RtlEnterCriticalSection( &PROFILE_CritSect );
 
-    if (PROFILE_Open( filename, FALSE )) {
-	if (section == NULL)
-            ret = PROFILE_GetSectionNames(buffer, len);
-	else 
-	    /* PROFILE_GetString can handle the 'entry == NULL' case */
-            ret = PROFILE_GetString( section, entry, def_val, buffer, len );
-    } else if (buffer && def_val) {
+    if (PROFILE_Open( filename, FALSE ))
+    {
+        PROFILEKEY *key = PROFILE_Find( &CurProfile->section, section, entry, FALSE, FALSE );
+        PROFILE_CopyEntry( buffer, (key && key->value) ? key->value : def_val, len );
+        TRACE("-> %s\n", debugstr_w( buffer ));
+        ret = strlenW( buffer );
+    }
+    else
+    {
        lstrcpynW( buffer, def_val, len );
        ret = strlenW( buffer );
     }
-    else
-       ret = 0;
 
     RtlLeaveCriticalSection( &PROFILE_CritSect );
 
@@ -1274,8 +1242,6 @@ UINT WINAPI GetPrivateProfileIntA( LPCSTR section, LPCSTR entry,
 INT WINAPI GetPrivateProfileSectionW( LPCWSTR section, LPWSTR buffer,
 				      DWORD len, LPCWSTR filename )
 {
-    int ret = 0;
-
     if (!section || !buffer)
     {
         SetLastError(ERROR_INVALID_PARAMETER);
@@ -1284,14 +1250,7 @@ INT WINAPI GetPrivateProfileSectionW( LPCWSTR section, LPWSTR buffer,
 
     TRACE("(%s, %p, %d, %s)\n", debugstr_w(section), buffer, len, debugstr_w(filename));
 
-    RtlEnterCriticalSection( &PROFILE_CritSect );
-
-    if (PROFILE_Open( filename, FALSE ))
-        ret = PROFILE_GetSection(CurProfile->section, section, buffer, len, TRUE);
-
-    RtlLeaveCriticalSection( &PROFILE_CritSect );
-
-    return ret;
+    return PROFILE_GetSection( filename, section, buffer, len, TRUE );
 }
 
 /***********************************************************************
@@ -1365,26 +1324,30 @@ BOOL WINAPI WritePrivateProfileStringW( LPCWSTR section, LPCWSTR entry,
 {
     BOOL ret = FALSE;
 
-    RtlEnterCriticalSection( &PROFILE_CritSect );
-
     if (!section && !entry && !string) /* documented "file flush" case */
     {
+        EnterCriticalSection( &PROFILE_CritSect );
         if (!filename || PROFILE_Open( filename, TRUE ))
         {
-            if (CurProfile) PROFILE_ReleaseFile();  /* always return FALSE in this case */
+            if (CurProfile) PROFILE_ReleaseFile();
         }
+        LeaveCriticalSection( &PROFILE_CritSect );
+        return FALSE;
     }
-    else if (PROFILE_Open( filename, TRUE ))
+    if (!entry) return PROFILE_DeleteSection( filename, section );
+
+    EnterCriticalSection( &PROFILE_CritSect );
+
+    if (PROFILE_Open( filename, TRUE ))
     {
-        if (!section) {
+        if (!section)
             SetLastError(ERROR_FILE_NOT_FOUND);
-        } else {
+        else
             ret = PROFILE_SetString( section, entry, string, FALSE);
-            if (ret) ret = PROFILE_FlushFile();
-        }
+        if (ret) ret = PROFILE_FlushFile();
     }
 
-    RtlLeaveCriticalSection( &PROFILE_CritSect );
+    LeaveCriticalSection( &PROFILE_CritSect );
     return ret;
 }
 
@@ -1424,36 +1387,40 @@ BOOL WINAPI WritePrivateProfileSectionW( LPCWSTR section,
     BOOL ret = FALSE;
     LPWSTR p;
 
-    RtlEnterCriticalSection( &PROFILE_CritSect );
-
     if (!section && !string)
     {
+        EnterCriticalSection( &PROFILE_CritSect );
         if (!filename || PROFILE_Open( filename, TRUE ))
         {
-            if (CurProfile) PROFILE_ReleaseFile();  /* always return FALSE in this case */
+            if (CurProfile) PROFILE_ReleaseFile();
         }
+        LeaveCriticalSection( &PROFILE_CritSect );
+        return FALSE;
     }
-    else if (PROFILE_Open( filename, TRUE )) {
-        if (!string) {/* delete the named section*/
-	    ret = PROFILE_SetString(section,NULL,NULL, FALSE);
-        } else {
-	    PROFILE_DeleteAllKeys(section);
-	    ret = TRUE;
-	    while(*string && ret) {
-                LPWSTR buf = HeapAlloc( GetProcessHeap(), 0, (strlenW(string)+1) * sizeof(WCHAR) );
-                strcpyW( buf, string );
-                if((p = strchrW( buf, '='))) {
-                    *p='\0';
-                    ret = PROFILE_SetString( section, buf, p+1, TRUE);
-                }
-                HeapFree( GetProcessHeap(), 0, buf );
-                string += strlenW(string)+1;
+    if (!string) return PROFILE_DeleteSection( filename, section );
+
+    EnterCriticalSection( &PROFILE_CritSect );
+
+    if (PROFILE_Open( filename, TRUE ))
+    {
+        PROFILE_DeleteAllKeys(section);
+        ret = TRUE;
+        while (*string && ret)
+        {
+            WCHAR *buf = HeapAlloc( GetProcessHeap(), 0, (strlenW( string ) + 1) * sizeof(WCHAR) );
+            strcpyW( buf, string );
+            if ((p = strchrW( buf, '=')))
+            {
+                *p = '\0';
+                ret = PROFILE_SetString( section, buf, p+1, TRUE );
             }
+            HeapFree( GetProcessHeap(), 0, buf );
+            string += strlenW( string ) + 1;
         }
         if (ret) ret = PROFILE_FlushFile();
     }
 
-    RtlLeaveCriticalSection( &PROFILE_CritSect );
+    LeaveCriticalSection( &PROFILE_CritSect );
     return ret;
 }
 
