@@ -20,9 +20,7 @@
 #define __WINE_NTDLL_MISC_H
 
 #include <stdarg.h>
-#include <signal.h>
 #include <sys/types.h>
-#include <pthread.h>
 
 #include "windef.h"
 #include "winnt.h"
@@ -40,20 +38,12 @@
 
 #define MAX_NT_PATH_LENGTH 277
 
-#define MAX_DOS_DRIVES 26
-
 #if defined(__i386__) || defined(__x86_64__) || defined(__arm__) || defined(__aarch64__)
 static const UINT_PTR page_size = 0x1000;
 #else
 extern UINT_PTR page_size DECLSPEC_HIDDEN;
 #endif
 
-struct drive_info
-{
-    dev_t dev;
-    ino_t ino;
-};
-
 extern NTSTATUS close_handle( HANDLE ) DECLSPEC_HIDDEN;
 
 /* exceptions */
@@ -72,26 +62,14 @@ extern LPCSTR debugstr_ObjectAttributes(const OBJECT_ATTRIBUTES *oa) DECLSPEC_HI
 /* init routines */
 extern void version_init(void) DECLSPEC_HIDDEN;
 extern void debug_init(void) DECLSPEC_HIDDEN;
-extern TEB *thread_init( SIZE_T *info_size, BOOL *suspend ) DECLSPEC_HIDDEN;
 extern void actctx_init(void) DECLSPEC_HIDDEN;
-extern void fill_cpu_info(void) DECLSPEC_HIDDEN;
 extern void heap_set_debug_flags( HANDLE handle ) DECLSPEC_HIDDEN;
 extern void init_unix_codepage(void) DECLSPEC_HIDDEN;
 extern void init_locale( HMODULE module ) DECLSPEC_HIDDEN;
-extern void init_user_process_params( SIZE_T data_size ) DECLSPEC_HIDDEN;
-extern char **build_envp( const WCHAR *envW ) DECLSPEC_HIDDEN;
+extern void init_user_process_params(void) DECLSPEC_HIDDEN;
 extern NTSTATUS restart_process( RTL_USER_PROCESS_PARAMETERS *params, NTSTATUS status ) DECLSPEC_HIDDEN;
 
-extern int __wine_main_argc;
-extern char **__wine_main_argv;
-extern WCHAR **__wine_main_wargv;
-
 /* server support */
-extern const char *build_dir DECLSPEC_HIDDEN;
-extern const char *data_dir DECLSPEC_HIDDEN;
-extern const char *config_dir DECLSPEC_HIDDEN;
-extern timeout_t server_start_time DECLSPEC_HIDDEN;
-extern unsigned int server_cpus DECLSPEC_HIDDEN;
 extern BOOL is_wow64 DECLSPEC_HIDDEN;
 extern NTSTATUS alloc_object_attributes( const OBJECT_ATTRIBUTES *attr, struct object_attributes **ret,
                                          data_size_t *ret_len ) DECLSPEC_HIDDEN;
@@ -114,11 +92,7 @@ extern const struct unix_funcs *unix_funcs DECLSPEC_HIDDEN;
 /* file I/O */
 extern NTSTATUS server_get_unix_name( HANDLE handle, ANSI_STRING *unix_name ) DECLSPEC_HIDDEN;
 extern void init_directories(void) DECLSPEC_HIDDEN;
-extern unsigned int DIR_get_drives_info( struct drive_info info[MAX_DOS_DRIVES] ) DECLSPEC_HIDDEN;
 
-/* virtual memory */
-extern void virtual_fill_image_information( const pe_image_info_t *pe_info,
-                                            SECTION_IMAGE_INFORMATION *info ) DECLSPEC_HIDDEN;
 extern struct _KUSER_SHARED_DATA *user_shared_data DECLSPEC_HIDDEN;
 
 /* locale */
@@ -153,7 +127,6 @@ struct ntdll_thread_data
     int                reply_fd;      /* fd for receiving server replies */
     int                wait_fd[2];    /* fd for sleeping server requests */
     BOOL               wow64_redir;   /* Wow64 filesystem redirection flag */
-    pthread_t          pthread_id;    /* pthread thread id */
 };
 
 C_ASSERT( sizeof(struct ntdll_thread_data) <= sizeof(((TEB *)0)->GdiTebBatch) );
@@ -163,8 +136,6 @@ static inline struct ntdll_thread_data *ntdll_get_thread_data(void)
     return (struct ntdll_thread_data *)&NtCurrentTeb()->GdiTebBatch;
 }
 
-extern SYSTEM_CPU_INFORMATION cpu_info DECLSPEC_HIDDEN;
-
 #define HASH_STRING_ALGORITHM_DEFAULT  0
 #define HASH_STRING_ALGORITHM_X65599   1
 #define HASH_STRING_ALGORITHM_INVALID  0xffffffff
@@ -176,51 +147,6 @@ void     WINAPI LdrInitializeThunk(CONTEXT*,void**,ULONG_PTR,ULONG_PTR);
 #define InterlockedCompareExchange64(dest,xchg,cmp) RtlInterlockedCompareExchange64(dest,xchg,cmp)
 #endif
 
-/* string functions */
-int    __cdecl NTDLL_tolower( int c );
-int    __cdecl _stricmp( LPCSTR str1, LPCSTR str2 );
-int    __cdecl NTDLL__wcsicmp( LPCWSTR str1, LPCWSTR str2 );
-int    __cdecl NTDLL__wcsnicmp( LPCWSTR str1, LPCWSTR str2, size_t n );
-int    __cdecl NTDLL_wcscmp( LPCWSTR str1, LPCWSTR str2 );
-int    __cdecl NTDLL_wcsncmp( LPCWSTR str1, LPCWSTR str2, size_t n );
-WCHAR  __cdecl NTDLL_towlower( WCHAR ch );
-WCHAR  __cdecl NTDLL_towupper( WCHAR ch );
-LPWSTR __cdecl NTDLL__wcslwr( LPWSTR str );
-LPWSTR __cdecl NTDLL__wcsupr( LPWSTR str );
-LPWSTR __cdecl NTDLL_wcscpy( LPWSTR dst, LPCWSTR src );
-LPWSTR __cdecl NTDLL_wcscat( LPWSTR dst, LPCWSTR src );
-LPWSTR __cdecl NTDLL_wcschr( LPCWSTR str, WCHAR ch );
-size_t __cdecl NTDLL_wcslen( LPCWSTR str );
-size_t __cdecl NTDLL_wcscspn( LPCWSTR str, LPCWSTR reject );
-LPWSTR __cdecl NTDLL_wcsncat( LPWSTR s1, LPCWSTR s2, size_t n );
-LPWSTR __cdecl NTDLL_wcsncpy( LPWSTR s1, LPCWSTR s2, size_t n );
-LPWSTR __cdecl NTDLL_wcspbrk( LPCWSTR str, LPCWSTR accept );
-LPWSTR __cdecl NTDLL_wcsrchr( LPCWSTR str, WCHAR ch );
-size_t __cdecl NTDLL_wcsspn( LPCWSTR str, LPCWSTR accept );
-LPWSTR __cdecl NTDLL_wcsstr( LPCWSTR str, LPCWSTR sub );
-LPWSTR __cdecl NTDLL_wcstok( LPWSTR str, LPCWSTR delim );
-LONG   __cdecl NTDLL_wcstol( LPCWSTR s, LPWSTR *end, INT base );
-ULONG  __cdecl NTDLL_wcstoul( LPCWSTR s, LPWSTR *end, INT base );
-int    WINAPIV NTDLL_swprintf( WCHAR *str, const WCHAR *format, ... );
-int    WINAPIV _snwprintf_s( WCHAR *str, SIZE_T size, SIZE_T len, const WCHAR *format, ... );
-
-#define wcsicmp(s1,s2) NTDLL__wcsicmp(s1,s2)
-#define wcsnicmp(s1,s2,n) NTDLL__wcsnicmp(s1,s2,n)
-#define towupper(c) NTDLL_towupper(c)
-#define wcslwr(s) NTDLL__wcslwr(s)
-#define wcsupr(s) NTDLL__wcsupr(s)
-#define wcscpy(d,s) NTDLL_wcscpy(d,s)
-#define wcscat(d,s) NTDLL_wcscat(d,s)
-#define wcschr(s,c) NTDLL_wcschr(s,c)
-#define wcspbrk(s,a) NTDLL_wcspbrk(s,a)
-#define wcsrchr(s,c) NTDLL_wcsrchr(s,c)
-#define wcstoul(s,e,b) NTDLL_wcstoul(s,e,b)
-#define wcslen(s) NTDLL_wcslen(s)
-#define wcscspn(s,r) NTDLL_wcscspn(s,r)
-#define wcsspn(s,a) NTDLL_wcsspn(s,a)
-#define wcscmp(s1,s2) NTDLL_wcscmp(s1,s2)
-#define wcsncmp(s1,s2,n) NTDLL_wcsncmp(s1,s2,n)
-
 /* convert from straight ASCII to Unicode without depending on the current codepage */
 static inline void ascii_to_unicode( WCHAR *dst, const char *src, size_t len )
 {
