@@ -26,19 +26,14 @@
   *	HKEY_CLASSES		\\REGISTRY\\MACHINE\\SOFTWARE\\CLASSES
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
-#include "wine/library.h"
 #include "ntdll_misc.h"
 #include "wine/debug.h"
-#include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(reg);
 
@@ -530,7 +525,7 @@ NTSTATUS WINAPI NtEnumerateValueKey( HANDLE handle, ULONG index,
  * NOTES
  *  the name in the KeyValueInformation is never set
  */
-NTSTATUS WINAPI NtQueryValueKey( HANDLE handle, const UNICODE_STRING *name,
+NTSTATUS WINAPI DECLSPEC_HOTPATCH NtQueryValueKey( HANDLE handle, const UNICODE_STRING *name,
                                  KEY_VALUE_INFORMATION_CLASS info_class,
                                  void *info, DWORD length, DWORD *result_len )
 {
@@ -1099,7 +1094,7 @@ static NTSTATUS RTL_ReportRegistryValue(PKEY_VALUE_FULL_INFORMATION pInfo,
                 for (offset = 0; offset <= pInfo->DataLength; offset += len + sizeof(WCHAR))
                     {
                     wstr = (WCHAR*)(((CHAR*)pInfo) + offset);
-                    len = strlenW(wstr) * sizeof(WCHAR);
+                    len = wcslen(wstr) * sizeof(WCHAR);
                     status = pQuery->QueryRoutine(pQuery->Name, pInfo->Type, wstr, len,
                         pContext, pQuery->EntryContext);
                     if(status != STATUS_SUCCESS && status != STATUS_BUFFER_TOO_SMALL)
@@ -1111,7 +1106,7 @@ static NTSTATUS RTL_ReportRegistryValue(PKEY_VALUE_FULL_INFORMATION pInfo,
                 while(count<=pInfo->DataLength)
                 {
                     String = (WCHAR*)(((CHAR*)pInfo) + pInfo->DataOffset)+count;
-                    count+=strlenW(String)+1;
+                    count+=wcslen(String)+1;
                     RtlInitUnicodeString(&src, (WCHAR*)(((CHAR*)pInfo) + pInfo->DataOffset));
                     res = 0;
                     dst.MaximumLength = 0;
@@ -1187,13 +1182,13 @@ static NTSTATUS RTL_KeyHandleCreateObject(ULONG RelativeTo, PCWSTR Path, POBJECT
         return STATUS_INVALID_PARAMETER;
     }
 
-    len = (strlenW(base) + strlenW(Path) + 1) * sizeof(WCHAR);
+    len = (wcslen(base) + wcslen(Path) + 1) * sizeof(WCHAR);
     str->Buffer = RtlAllocateHeap(GetProcessHeap(), 0, len);
     if (str->Buffer == NULL)
         return STATUS_NO_MEMORY;
 
-    strcpyW(str->Buffer, base);
-    strcatW(str->Buffer, Path);
+    wcscpy(str->Buffer, base);
+    wcscat(str->Buffer, Path);
     str->Length = len - sizeof(WCHAR);
     str->MaximumLength = len;
     InitializeObjectAttributes(regkey, str, OBJ_CASE_INSENSITIVE, NULL, NULL);
