@@ -516,6 +516,15 @@ static inline void set_fs( WORD val ) { __asm__( "mov %0,%%fs" :: "r" (val)); }
 static inline void set_gs( WORD val ) { __asm__( "mov %0,%%gs" :: "r" (val)); }
 
 
+/***********************************************************************
+ *           unwind_builtin_dll
+ */
+NTSTATUS CDECL unwind_builtin_dll( ULONG type, struct _DISPATCHER_CONTEXT *dispatch, CONTEXT *context )
+{
+    return STATUS_UNSUCCESSFUL;
+}
+
+
 /***********************************************************************
  *           is_gdt_sel
  */
@@ -1579,6 +1588,10 @@ static void setup_raise_exception( ucontext_t *sigcontext, struct stack_layout *
     stack->context_ptr  = &stack->context;
 }
 
+void WINAPI call_user_exception_dispatcher( EXCEPTION_RECORD *rec, CONTEXT *context )
+{
+    pKiUserExceptionDispatcher( rec, context );
+}
 
 /**********************************************************************
  *		get_fpu_code
@@ -1921,16 +1934,7 @@ struct ldt_copy
 } __wine_ldt_copy;
 
 static WORD gdt_fs_sel;
-
-static RTL_CRITICAL_SECTION ldt_section;
-static RTL_CRITICAL_SECTION_DEBUG critsect_debug =
-{
-    0, 0, &ldt_section,
-    { &critsect_debug.ProcessLocksList, &critsect_debug.ProcessLocksList },
-      0, 0, { (DWORD_PTR)(__FILE__ ": ldt_section") }
-};
-static RTL_CRITICAL_SECTION ldt_section = { &critsect_debug, -1, 0, 0, 0, 0 };
-
+static pthread_mutex_t ldt_mutex = PTHREAD_MUTEX_INITIALIZER;
 static const LDT_ENTRY null_entry;
 
 static inline void *ldt_get_base( LDT_ENTRY ent )
@@ -2097,10 +2101,10 @@ NTSTATUS WINAPI NtSetLdtEntries( ULONG sel1, LDT_ENTRY entry1, ULONG sel2, LDT_E
     if (sel1 && (sel1 >> 3) < first_ldt_entry) return STATUS_INVALID_LDT_DESCRIPTOR;
     if (sel2 && (sel2 >> 3) < first_ldt_entry) return STATUS_INVALID_LDT_DESCRIPTOR;
 
-    server_enter_uninterrupted_section( &ldt_section, &sigset );
+    server_enter_uninterrupted_section( &ldt_mutex, &sigset );
     if (sel1) ldt_set_entry( sel1, entry1 );
     if (sel2) ldt_set_entry( sel2, entry2 );
-    server_leave_uninterrupted_section( &ldt_section, &sigset );
+    server_leave_uninterrupted_section( &ldt_mutex, &sigset );
    return STATUS_SUCCESS;
 }
 
@@ -2152,14 +2156,14 @@ NTSTATUS signal_alloc_thread( TEB *teb )
         }
         else
         {
-            server_enter_uninterrupted_section( &ldt_section, &sigset );
+            server_enter_uninterrupted_section( &ldt_mutex, &sigset );
             for (idx = first_ldt_entry; idx < LDT_SIZE; idx++)
             {
                 if (__wine_ldt_copy.flags[idx]) continue;
                 ldt_set_entry( (idx << 3) | 7, entry );
                 break;
             }
-            server_leave_uninterrupted_section( &ldt_section, &sigset );
+            server_leave_uninterrupted_section( &ldt_mutex, &sigset );
             if (idx == LDT_SIZE) return STATUS_TOO_MANY_THREADS;
         }
         thread_data->fs = (idx << 3) | 7;
@@ -2180,9 +2184,9 @@ void signal_free_thread( TEB *teb )
 
     if (gdt_fs_sel) return;
 
-    server_enter_uninterrupted_section( &ldt_section, &sigset );
+    server_enter_uninterrupted_section( &ldt_mutex, &sigset );
     __wine_ldt_copy.flags[thread_data->fs >> 3] = 0;
-    server_leave_uninterrupted_section( &ldt_section, &sigset );
+    server_leave_uninterrupted_section( &ldt_mutex, &sigset );
 }
 
 
