@@ -29,6 +29,8 @@
 #include <errno.h>
 #include <limits.h>
 #include <stdarg.h>
+#include <stdio.h>
+#include <string.h>
 #include <pthread.h>
 #include <signal.h>
 #include <sys/types.h>
@@ -58,7 +60,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(seh);
 #define PTHREAD_STACK_MIN 16384
 #endif
 
-static int *nb_threads;
+static int nb_threads = 1;
 
 static inline int get_unix_exit_code( NTSTATUS status )
 {
@@ -81,40 +83,6 @@ static void pthread_exit_wrapper( int status )
 }
 
 
-/***********************************************************************
- *           init_threading
- */
-TEB * CDECL init_threading( int *nb_threads_ptr, struct ldt_copy **ldt_copy, SIZE_T *size, BOOL *suspend,
-                            unsigned int *cpus, BOOL *wow64, timeout_t *start_time )
-{
-    TEB *teb;
-    SIZE_T info_size;
-#ifdef __i386__
-    extern struct ldt_copy __wine_ldt_copy;
-    *ldt_copy = &__wine_ldt_copy;
-#endif
-    nb_threads = nb_threads_ptr;
-
-    teb = virtual_alloc_first_teb();
-
-    signal_init_threading();
-    signal_alloc_thread( teb );
-    signal_init_thread( teb );
-    dbg_init();
-    server_init_process();
-    info_size = server_init_thread( teb->Peb, suspend );
-    virtual_map_user_shared_data();
-    init_files();
-    NtCreateKeyedEvent( &keyed_event, GENERIC_READ | GENERIC_WRITE, NULL, 0 );
-
-    if (size) *size = info_size;
-    if (cpus) *cpus = server_cpus;
-    if (wow64) *wow64 = is_wow64;
-    if (start_time) *start_time = server_start_time;
-    return teb;
-}
-
-
 /* info passed to a starting thread */
 struct startup_info
 {
@@ -225,7 +193,7 @@ NTSTATUS WINAPI NtCreateThreadEx( HANDLE *handle, ACCESS_MASK access, OBJECT_ATT
 
     if (server_pipe( request_pipe ) == -1)
     {
-        RtlFreeHeap( GetProcessHeap(), 0, objattr );
+        free( objattr );
         return STATUS_TOO_MANY_OPENED_FILES;
     }
     server_send_fd( request_pipe[0] );
@@ -248,7 +216,7 @@ NTSTATUS WINAPI NtCreateThreadEx( HANDLE *handle, ACCESS_MASK access, OBJECT_ATT
     }
     SERVER_END_REQ;
 
-    RtlFreeHeap( GetProcessHeap(), 0, objattr );
+    free( objattr );
     if (status)
     {
         close( request_pipe[1] );
@@ -289,10 +257,10 @@ NTSTATUS WINAPI NtCreateThreadEx( HANDLE *handle, ACCESS_MASK access, OBJECT_ATT
                            (char *)teb->Tib.StackBase + extra_stack - (char *)teb->DeallocationStack );
     pthread_attr_setguardsize( &pthread_attr, 0 );
     pthread_attr_setscope( &pthread_attr, PTHREAD_SCOPE_SYSTEM ); /* force creating a kernel thread */
-    InterlockedIncrement( nb_threads );
+    InterlockedIncrement( &nb_threads );
     if (pthread_create( &pthread_id, &pthread_attr, (void * (*)(void *))start_thread, teb ))
     {
-        InterlockedDecrement( nb_threads );
+        InterlockedDecrement( &nb_threads );
         virtual_free_teb( teb );
         status = STATUS_NO_MEMORY;
     }
@@ -318,7 +286,7 @@ done:
 void abort_thread( int status )
 {
     pthread_sigmask( SIG_BLOCK, &server_block_set, NULL );
-    if (InterlockedDecrement( nb_threads ) <= 0) abort_process( status );
+    if (InterlockedDecrement( &nb_threads ) <= 0) abort_process( status );
     signal_exit_thread( status, pthread_exit_wrapper );
 }
 
@@ -447,7 +415,7 @@ NTSTATUS WINAPI NtRaiseException( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL
     if (status == DBG_CONTINUE || status == DBG_EXCEPTION_HANDLED)
         NtSetContextThread( GetCurrentThread(), context );
 
-    if (first_chance) pKiUserExceptionDispatcher( rec, context );
+    if (first_chance) call_user_exception_dispatcher( rec, context );
 
     if (rec->ExceptionFlags & EH_STACK_INVALID)
         ERR("Exception frame is not in stack limits => unable to dispatch exception.\n");
@@ -822,6 +790,62 @@ static void wow64_context_to_server( context_t *to, const WOW64_CONTEXT *from )
 
 #endif /* __x86_64__ */
 
+#ifdef linux
+BOOL get_thread_times(int unix_pid, int unix_tid, LARGE_INTEGER *kernel_time, LARGE_INTEGER *user_time)
+{
+    unsigned long clocks_per_sec = sysconf( _SC_CLK_TCK );
+    unsigned long usr, sys;
+    const char *pos;
+    char buf[512];
+    FILE *f;
+    int i;
+
+    if (unix_tid == -1)
+        sprintf( buf, "/proc/%u/stat", unix_pid );
+    else
+        sprintf( buf, "/proc/%u/task/%u/stat", unix_pid, unix_tid );
+    if (!(f = fopen( buf, "r" )))
+    {
+        ERR("Failed to open %s: %s\n", buf, strerror(errno));
+        return FALSE;
+    }
+
+    pos = fgets( buf, sizeof(buf), f );
+    fclose( f );
+
+    /* the process name is printed unescaped, so we have to skip to the last ')'
+     * to avoid misinterpreting the string */
+    if (pos) pos = strrchr( pos, ')' );
+    if (pos) pos = strchr( pos + 1, ' ' );
+    if (pos) pos++;
+
+    /* skip over the following fields: state, ppid, pgid, sid, tty_nr, tty_pgrp,
+     * task->flags, min_flt, cmin_flt, maj_flt, cmaj_flt */
+    for (i = 0; i < 11 && pos; i++)
+    {
+        pos = strchr( pos + 1, ' ' );
+        if (pos) pos++;
+    }
+
+    /* the next two values are user and system time */
+    if (pos && (sscanf( pos, "%lu %lu", &usr, &sys ) == 2))
+    {
+        kernel_time->QuadPart = (ULONGLONG)sys * 10000000 / clocks_per_sec;
+        user_time->QuadPart = (ULONGLONG)usr * 10000000 / clocks_per_sec;
+        return TRUE;
+    }
+
+    ERR("Failed to parse %s\n", debugstr_a(buf));
+    return FALSE;
+}
+#else
+BOOL get_thread_times(int unix_pid, int unix_tid, LARGE_INTEGER *kernel_time, LARGE_INTEGER *user_time)
+{
+    static int once;
+    if (!once++) FIXME("not implemented on this platform\n");
+    return FALSE;
+}
+#endif
 
 /******************************************************************************
  *              NtQueryInformationThread  (NTDLL.@)
@@ -831,6 +855,8 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
 {
     NTSTATUS status;
 
+    TRACE("(%p,%d,%p,%x,%p)\n", handle, class, data, length, ret_len);
+
     switch (class)
     {
     case ThreadBasicInformation:
@@ -841,7 +867,6 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
         SERVER_START_REQ( get_thread_info )
         {
             req->handle = wine_server_obj_handle( handle );
-            req->tid_in = 0;
             if (!(status = wine_server_call( req )))
             {
                 info.ExitStatus             = reply->exit_code;
@@ -870,7 +895,7 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
         SERVER_START_REQ( get_thread_info )
         {
             req->handle = wine_server_obj_handle( handle );
-            req->tid_in = 0;
+            req->access = THREAD_QUERY_INFORMATION;
             if (!(status = wine_server_call( req ))) affinity = reply->affinity & affinity_mask;
         }
         SERVER_END_REQ;
@@ -885,6 +910,7 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
     case ThreadTimes:
     {
         KERNEL_USER_TIMES kusrt;
+        int unix_pid, unix_tid;
 
         SERVER_START_REQ( get_thread_times )
         {
@@ -894,15 +920,21 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
             {
                 kusrt.CreateTime.QuadPart = reply->creation_time;
                 kusrt.ExitTime.QuadPart = reply->exit_time;
+                unix_pid = reply->unix_pid;
+                unix_tid = reply->unix_tid;
             }
         }
         SERVER_END_REQ;
         if (status == STATUS_SUCCESS)
         {
-            /* We call times(2) for kernel time or user time */
-            /* We can only (portably) do this for the current thread */
-            if (handle == GetCurrentThread())
+            BOOL ret = FALSE;
+
+            kusrt.KernelTime.QuadPart = kusrt.UserTime.QuadPart = 0;
+            if (unix_pid != -1 && unix_tid != -1)
+                ret = get_thread_times( unix_pid, unix_tid, &kusrt.KernelTime, &kusrt.UserTime );
+            if (!ret && handle == GetCurrentThread())
             {
+                /* fall back to process times */
                 struct tms time_buf;
                 long clocks_per_sec = sysconf(_SC_CLK_TCK);
 
@@ -910,20 +942,6 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
                 kusrt.KernelTime.QuadPart = (ULONGLONG)time_buf.tms_stime * 10000000 / clocks_per_sec;
                 kusrt.UserTime.QuadPart = (ULONGLONG)time_buf.tms_utime * 10000000 / clocks_per_sec;
             }
-            else
-            {
-                static BOOL reported = FALSE;
-
-                kusrt.KernelTime.QuadPart = 0;
-                kusrt.UserTime.QuadPart = 0;
-                if (reported)
-                    TRACE("Cannot get kerneltime or usertime of other threads\n");
-                else
-                {
-                    FIXME("Cannot get kerneltime or usertime of other threads\n");
-                    reported = TRUE;
-                }
-            }
             if (data) memcpy( data, &kusrt, min( length, sizeof(kusrt) ));
             if (ret_len) *ret_len = min( length, sizeof(kusrt) );
         }
@@ -935,16 +953,16 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
 
     case ThreadAmILastThread:
     {
+        if (length != sizeof(ULONG)) return STATUS_INFO_LENGTH_MISMATCH;
         SERVER_START_REQ( get_thread_info )
         {
             req->handle = wine_server_obj_handle( handle );
-            req->tid_in = 0;
             status = wine_server_call( req );
             if (status == STATUS_SUCCESS)
             {
-                BOOLEAN last = reply->last;
-                if (data) memcpy( data, &last, min( length, sizeof(last) ));
-                if (ret_len) *ret_len = min( length, sizeof(last) );
+                ULONG last = reply->last;
+                if (data) memcpy( data, &last, sizeof(last) );
+                if (ret_len) *ret_len = sizeof(last);
             }
         }
         SERVER_END_REQ;
@@ -956,7 +974,7 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
         SERVER_START_REQ( get_thread_info )
         {
             req->handle = wine_server_obj_handle( handle );
-            req->tid_in = 0;
+            req->access = THREAD_QUERY_INFORMATION;
             status = wine_server_call( req );
             if (status == STATUS_SUCCESS)
             {
@@ -980,7 +998,6 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
         SERVER_START_REQ( get_thread_info )
         {
             req->handle = wine_server_obj_handle( handle );
-            req->tid_in = 0;
             if (!(status = wine_server_call( req ))) affinity.Mask = reply->affinity & affinity_mask;
         }
         SERVER_END_REQ;
@@ -1007,7 +1024,6 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
         SERVER_START_REQ( get_thread_info )
         {
             req->handle = wine_server_obj_handle( handle );
-            req->tid_in = 0;
             if (!(status = wine_server_call( req ))) *(ULONG *)data = reply->suspend_count;
         }
         SERVER_END_REQ;
@@ -1063,6 +1079,20 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
 #endif
     }
 
+    case ThreadHideFromDebugger:
+        if (length != sizeof(BOOLEAN)) return STATUS_INFO_LENGTH_MISMATCH;
+        if (!data) return STATUS_ACCESS_VIOLATION;
+        SERVER_START_REQ( get_thread_info )
+        {
+            req->handle = wine_server_obj_handle( handle );
+            req->access = THREAD_QUERY_INFORMATION;
+            if ((status = wine_server_call( req ))) return status;
+            *(BOOLEAN*)data = reply->dbg_hidden;
+        }
+        SERVER_END_REQ;
+        if (ret_len) *ret_len = sizeof(BOOLEAN);
+        return STATUS_SUCCESS;
+
     case ThreadPriority:
     case ThreadBasePriority:
     case ThreadImpersonationToken:
@@ -1088,6 +1118,8 @@ NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
 {
     NTSTATUS status;
 
+    TRACE("(%p,%d,%p,%x)\n", handle, class, data, length);
+
     switch (class)
     {
     case ThreadZeroTlsCell:
@@ -1152,8 +1184,15 @@ NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
     }
 
     case ThreadHideFromDebugger:
-        /* pretend the call succeeded to satisfy some code protectors */
-        return STATUS_SUCCESS;
+        if (length) return STATUS_INFO_LENGTH_MISMATCH;
+        SERVER_START_REQ( set_thread_info )
+        {
+            req->handle = wine_server_obj_handle( handle );
+            req->mask   = SET_THREAD_INFO_DBG_HIDDEN;
+            status = wine_server_call( req );
+        }
+        SERVER_END_REQ;
+        return status;
 
     case ThreadQuerySetWin32StartAddress:
     {
