@@ -18,15 +18,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <assert.h>
 #include <errno.h>
-#include <fcntl.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -201,39 +194,6 @@ NTSTATUS WINAPI NtUnmapViewOfSection( HANDLE process, PVOID addr )
 }
 
 
-/******************************************************************************
- *             virtual_fill_image_information
- *
- * Helper for NtQuerySection.
- */
-void virtual_fill_image_information( const pe_image_info_t *pe_info, SECTION_IMAGE_INFORMATION *info )
-{
-    info->TransferAddress      = wine_server_get_ptr( pe_info->entry_point );
-    info->ZeroBits             = pe_info->zerobits;
-    info->MaximumStackSize     = pe_info->stack_size;
-    info->CommittedStackSize   = pe_info->stack_commit;
-    info->SubSystemType        = pe_info->subsystem;
-    info->SubsystemVersionLow  = pe_info->subsystem_low;
-    info->SubsystemVersionHigh = pe_info->subsystem_high;
-    info->GpValue              = pe_info->gp;
-    info->ImageCharacteristics = pe_info->image_charact;
-    info->DllCharacteristics   = pe_info->dll_charact;
-    info->Machine              = pe_info->machine;
-    info->ImageContainsCode    = pe_info->contains_code;
-    info->u.ImageFlags         = pe_info->image_flags & ~(IMAGE_FLAGS_WineBuiltin|IMAGE_FLAGS_WineFakeDll);
-    info->LoaderFlags          = pe_info->loader_flags;
-    info->ImageFileSize        = pe_info->file_size;
-    info->CheckSum             = pe_info->checksum;
-#ifndef _WIN64 /* don't return 64-bit values to 32-bit processes */
-    if (pe_info->machine == IMAGE_FILE_MACHINE_AMD64 || pe_info->machine == IMAGE_FILE_MACHINE_ARM64)
-    {
-        info->TransferAddress = (void *)0x81231234;  /* sic */
-        info->MaximumStackSize = 0x100000;
-        info->CommittedStackSize = 0x10000;
-    }
-#endif
-}
-
 /******************************************************************************
  *             NtQuerySection   (NTDLL.@)
  *             ZwQuerySection   (NTDLL.@)
