@@ -35,6 +35,8 @@
 
 #include "driver.h"
 
+#include "utils.h"
+
 static const WCHAR device_name[] = {'\\','D','e','v','i','c','e',
                                     '\\','W','i','n','e','T','e','s','t','D','r','i','v','e','r',0};
 static const WCHAR upper_name[] = {'\\','D','e','v','i','c','e',
@@ -45,17 +47,6 @@ static const WCHAR driver_link[] = {'\\','D','o','s','D','e','v','i','c','e','s'
 static DRIVER_OBJECT *driver_obj;
 static DEVICE_OBJECT *lower_device, *upper_device;
 
-static HANDLE okfile;
-static LONG successes;
-static LONG failures;
-static LONG skipped;
-static LONG todo_successes;
-static LONG todo_failures;
-static int todo_level, todo_do_loop;
-static int running_under_wine;
-static int winetest_debug;
-static int winetest_report_success;
-
 static POBJECT_TYPE *pExEventObjectType, *pIoFileObjectType, *pPsThreadType, *pIoDriverObjectType;
 static PEPROCESS *pPsInitialSystemProcess;
 static void *create_caller_thread;
@@ -64,133 +55,6 @@ static PETHREAD create_irp_thread;
 
 NTSTATUS WINAPI ZwQueryInformationProcess(HANDLE,PROCESSINFOCLASS,void*,ULONG,ULONG*);
 
-static void kvprintf(const char *format, __ms_va_list ap)
-{
-    static char buffer[512];
-    IO_STATUS_BLOCK io;
-    int len = vsnprintf(buffer, sizeof(buffer), format, ap);
-    ZwWriteFile(okfile, NULL, NULL, NULL, &io, buffer, len, NULL, NULL);
-}
-
-static void WINAPIV kprintf(const char *format, ...)
-{
-    __ms_va_list valist;
-
-    __ms_va_start(valist, format);
-    kvprintf(format, valist);
-    __ms_va_end(valist);
-}
-
-static void WINAPIV vok_(const char *file, int line, int condition, const char *msg,  __ms_va_list args)
-{
-    const char *current_file;
-
-    if (!(current_file = drv_strrchr(file, '/')) &&
-        !(current_file = drv_strrchr(file, '\\')))
-        current_file = file;
-    else
-        current_file++;
-
-    if (todo_level)
-    {
-        if (condition)
-        {
-            kprintf("%s:%d: Test succeeded inside todo block: ", current_file, line);
-            kvprintf(msg, args);
-            InterlockedIncrement(&todo_failures);
-        }
-        else
-        {
-            if (winetest_debug > 0)
-            {
-                kprintf("%s:%d: Test marked todo: ", current_file, line);
-                kvprintf(msg, args);
-            }
-            InterlockedIncrement(&todo_successes);
-        }
-    }
-    else
-    {
-        if (!condition)
-        {
-            kprintf("%s:%d: Test failed: ", current_file, line);
-            kvprintf(msg, args);
-            InterlockedIncrement(&failures);
-        }
-        else
-        {
-            if (winetest_report_success)
-                kprintf("%s:%d: Test succeeded\n", current_file, line);
-            InterlockedIncrement(&successes);
-        }
-    }
-}
-
-static void WINAPIV ok_(const char *file, int line, int condition, const char *msg, ...)
-{
-    __ms_va_list args;
-    __ms_va_start(args, msg);
-    vok_(file, line, condition, msg, args);
-    __ms_va_end(args);
-}
-
-static void vskip_(const char *file, int line, const char *msg, __ms_va_list args)
-{
-    const char *current_file;
-
-    if (!(current_file = drv_strrchr(file, '/')) &&
-        !(current_file = drv_strrchr(file, '\\')))
-        current_file = file;
-    else
-        current_file++;
-
-    kprintf("%s:%d: Tests skipped: ", current_file, line);
-    kvprintf(msg, args);
-    skipped++;
-}
-
-static void WINAPIV win_skip_(const char *file, int line, const char *msg, ...)
-{
-    __ms_va_list args;
-    __ms_va_start(args, msg);
-    if (running_under_wine)
-        vok_(file, line, 0, msg, args);
-    else
-        vskip_(file, line, msg, args);
-    __ms_va_end(args);
-}
-
-static void winetest_start_todo( int is_todo )
-{
-    todo_level = (todo_level << 1) | (is_todo != 0);
-    todo_do_loop=1;
-}
-
-static int winetest_loop_todo(void)
-{
-    int do_loop=todo_do_loop;
-    todo_do_loop=0;
-    return do_loop;
-}
-
-static void winetest_end_todo(void)
-{
-    todo_level >>= 1;
-}
-
-static int broken(int condition)
-{
-    return !running_under_wine && condition;
-}
-
-#define ok(condition, ...)  ok_(__FILE__, __LINE__, condition, __VA_ARGS__)
-#define todo_if(is_todo) for (winetest_start_todo(is_todo); \
-                              winetest_loop_todo(); \
-                              winetest_end_todo())
-#define todo_wine               todo_if(running_under_wine)
-#define todo_wine_if(is_todo)   todo_if((is_todo) && running_under_wine)
-#define win_skip(...)           win_skip_(__FILE__, __LINE__, __VA_ARGS__)
-
 static void *get_proc_address(const char *name)
 {
     UNICODE_STRING name_u;
@@ -1706,6 +1570,33 @@ static void test_stack_limits(void)
     ok(low < (ULONG_PTR)&low && (ULONG_PTR)&low < high, "stack variable is not in stack limits\n");
 }
 
+static unsigned int got_completion;
+
+static NTSTATUS WINAPI completion_cb(DEVICE_OBJECT *device, IRP *irp, void *context)
+{
+    ok(device == context, "Got device %p; expected %p.\n", device, context);
+    ++got_completion;
+    return STATUS_SUCCESS;
+}
+
+static void test_completion(void)
+{
+    IO_STATUS_BLOCK io;
+    NTSTATUS ret;
+    KEVENT event;
+    IRP *irp;
+
+    KeInitializeEvent(&event, NotificationEvent, FALSE);
+
+    irp = IoBuildDeviceIoControlRequest(IOCTL_WINETEST_COMPLETION, upper_device,
+            NULL, 0, NULL, 0, FALSE, &event, &io);
+
+    IoSetCompletionRoutine(irp, completion_cb, NULL, TRUE, TRUE, TRUE);
+    ret = IoCallDriver(upper_device, irp);
+    ok(ret == STATUS_SUCCESS, "IoCallDriver returned %#x\n", ret);
+    ok(got_completion == 2, "got %u calls to completion routine\n");
+}
+
 static void test_IoAttachDeviceToDeviceStack(void)
 {
     DEVICE_OBJECT *dev1, *dev2, *dev3, *ret;
@@ -1850,6 +1741,7 @@ static void WINAPI main_test_task(DEVICE_OBJECT *device, void *context)
     test_call_driver(device);
     test_cancel_irp(device);
     test_stack_limits();
+    test_completion();
 
     /* print process report */
     if (winetest_debug)
@@ -2310,6 +2202,23 @@ static NTSTATUS test_mismatched_status_ioctl(IRP *irp, IO_STACK_LOCATION *stack,
     return STATUS_SUCCESS;
 }
 
+static NTSTATUS test_completion_ioctl(DEVICE_OBJECT *device, IRP *irp)
+{
+    if (device == upper_device)
+    {
+        IoCopyCurrentIrpStackLocationToNext(irp);
+        IoSetCompletionRoutine(irp, completion_cb, upper_device, TRUE, TRUE, TRUE);
+        return IoCallDriver(lower_device, irp);
+    }
+    else
+    {
+        ok(device == lower_device, "Got wrong device.\n");
+        irp->IoStatus.Status = STATUS_SUCCESS;
+        IoCompleteRequest(irp, IO_NO_INCREMENT);
+        return STATUS_SUCCESS;
+    }
+}
+
 static NTSTATUS WINAPI driver_Create(DEVICE_OBJECT *device, IRP *irp)
 {
     IO_STACK_LOCATION *irpsp = IoGetCurrentIrpStackLocation( irp );
@@ -2373,6 +2282,8 @@ static NTSTATUS WINAPI driver_IoControl(DEVICE_OBJECT *device, IRP *irp)
             break;
         case IOCTL_WINETEST_MISMATCHED_STATUS:
             return test_mismatched_status_ioctl(irp, stack, &irp->IoStatus.Information);
+        case IOCTL_WINETEST_COMPLETION:
+            return test_completion_ioctl(device, irp);
         default:
             break;
     }
