@@ -106,6 +106,7 @@ typedef struct dds_info {
     DXGI_FORMAT format;
     WICDdsDimension dimension;
     WICDdsAlphaMode alpha_mode;
+    const GUID *pixel_format;
 } dds_info;
 
 typedef struct dds_frame_info {
@@ -117,6 +118,7 @@ typedef struct dds_frame_info {
     UINT block_height;
     UINT width_in_blocks;
     UINT height_in_blocks;
+    const GUID *pixel_format;
 } dds_frame_info;
 
 typedef struct DdsDecoder {
@@ -229,6 +231,8 @@ static void get_dds_info(dds_info* info, DDS_HEADER *header, DDS_HEADER_DXT10 *h
         info->dimension = get_dimension(header, NULL);
         info->alpha_mode = get_alpha_mode_from_fourcc(header->ddspf.fourCC);
     }
+    info->pixel_format = (info->alpha_mode == WICDdsAlphaModePremultiplied) ?
+                         &GUID_WICPixelFormat32bppPBGRA : &GUID_WICPixelFormat32bppBGRA;
 
     /* get frame count */
     if (info->depth == 1) {
@@ -352,9 +356,15 @@ static HRESULT WINAPI DdsFrameDecode_GetSize(IWICBitmapFrameDecode *iface,
 static HRESULT WINAPI DdsFrameDecode_GetPixelFormat(IWICBitmapFrameDecode *iface,
                                                     WICPixelFormatGUID *pPixelFormat)
 {
-    FIXME("(%p,%p): stub.\n", iface, pPixelFormat);
+    DdsFrameDecode *This = impl_from_IWICBitmapFrameDecode(iface);
 
-    return E_NOTIMPL;
+    if (!pPixelFormat) return E_INVALIDARG;
+
+    *pPixelFormat = *This->info.pixel_format;
+
+    TRACE("(%p) -> %s\n", iface, debugstr_guid(pPixelFormat));
+
+    return S_OK;
 }
 
 static HRESULT WINAPI DdsFrameDecode_GetResolution(IWICBitmapFrameDecode *iface,
@@ -918,6 +928,7 @@ static HRESULT WINAPI DdsDecoder_Dds_GetFrame(IWICDdsDecoder *iface,
     frame_decode->info.block_height = DDS_BLOCK_HEIGHT;
     frame_decode->info.width_in_blocks = frame_width_in_blocks;
     frame_decode->info.height_in_blocks = frame_height_in_blocks;
+    frame_decode->info.pixel_format = This->info.pixel_format;
     frame_decode->data = HeapAlloc(GetProcessHeap(), 0, frame_size);
     hr = IStream_Seek(This->stream, seek, SEEK_SET, NULL);
     if (hr != S_OK) goto end;
